@@ -1,7 +1,7 @@
 import subprocess, site, importlib
 from pathlib import Path
 from python_helper import Constant as c
-from python_helper import log, StringHelper, SettingHelper, EnvironmentHelper, ObjectHelper
+from python_helper import log, StringHelper, SettingHelper, EnvironmentHelper, ObjectHelper, ReflectionHelper
 
 global GLOBALS
 GLOBALS = None
@@ -379,11 +379,11 @@ class Globals:
         self.settingTree = self.getEnvironmentSettingTree(defaultSettingFilePath=self.defaultSettingFilePath, settingTree=self.settingTree)
 
     def getDefaultSettingTree(self) :
-        self.defaultSettingFilePath = f'{self.apiPath}{Globals.API_BACK_SLASH}{Globals.RESOURCE_BACK_SLASH}{self.defaultSettingFileName}.{Globals.EXTENSION}'
+        self.defaultSettingFilePath = f'{self.apiPath}{Globals.API_BACK_SLASH}{Globals.RESOURCE_BACK_SLASH}{self.defaultSettingFileName}{c.DOT}{Globals.EXTENSION}'
         return self.getSettingTree(settingFilePath=self.defaultSettingFilePath)
 
     def getEnvironmentSettingTree(self, defaultSettingFilePath=None, settingTree=None) :
-        self.settingFilePath = f'{self.apiPath}{Globals.API_BACK_SLASH}{Globals.RESOURCE_BACK_SLASH}{self.settingsFileName}.{Globals.EXTENSION}'
+        self.settingFilePath = f'{self.apiPath}{Globals.API_BACK_SLASH}{Globals.RESOURCE_BACK_SLASH}{self.settingsFileName}{c.DOT}{Globals.EXTENSION}'
         return self.getSettingTree(settingFilePath=self.settingFilePath, defaultSettingFilePath=defaultSettingFilePath, settingTree=settingTree)
 
     def getSettingTree(self, settingFilePath=None, defaultSettingFilePath=None, settingTree=None) :
@@ -500,20 +500,28 @@ class Globals:
         return extension
 
     def getStaticPackagePath(self) :
-        staticPackageList = site.getsitepackages()
-        self.log(f'Static packages list: {StringHelper.prettyJson(staticPackageList)}. Picking the first one')
-        staticPackage = str(staticPackageList[0])
-        staticPackage = staticPackage.replace(f'{c.BACK_SLASH}{c.BACK_SLASH}',Globals.OS_SEPARATOR)
-        staticPackage = staticPackage.replace(c.BACK_SLASH,Globals.OS_SEPARATOR)
-        staticPackage = staticPackage.replace(f'{c.SLASH}{c.SLASH}',Globals.OS_SEPARATOR)
-        staticPackage = staticPackage.replace(c.SLASH,Globals.OS_SEPARATOR)
-        if staticPackage[-1] == str(Globals.OS_SEPARATOR) :
-            staticPackage = staticPackage[:-1]
-        herokuPythonLibPath = Globals.HEROKU_PYTHON.replace(Globals.TOKEN_PYTHON_VERSION, str(self.getSetting(AttributeKey.PYTHON_VERSION)))
-        if staticPackage.endswith(herokuPythonLibPath) :
-            staticPackage = staticPackage.replace(herokuPythonLibPath,c.NOTHING)
-        staticPackage = f'{staticPackage}{Globals.STATIC_PACKAGE_PATH}'
-        self.setting(f'Static package: "{staticPackage}"')
+        staticPackage = self.getSetting(AttributeKey.PYTHON_STATIC_PACKAGE)
+        if ObjectHelper.isNone(staticPackage) :
+            if EnvironmentHelper.isLinux() == 'linux' :
+                staticPackage = str(site.getusersitepackages())
+                self.log(f'Static package (before handlings): "{staticPackage}"')
+            else :
+                staticPackageList = site.getsitepackages()
+                self.log(f'Static packages list: {StringHelper.prettyJson(staticPackageList)}. Picking the first one')
+                staticPackage = str(staticPackageList[0])
+            staticPackage = staticPackage.replace(f'{c.BACK_SLASH}{c.BACK_SLASH}',Globals.OS_SEPARATOR)
+            staticPackage = staticPackage.replace(c.BACK_SLASH,Globals.OS_SEPARATOR)
+            staticPackage = staticPackage.replace(f'{c.SLASH}{c.SLASH}',Globals.OS_SEPARATOR)
+            staticPackage = staticPackage.replace(c.SLASH,Globals.OS_SEPARATOR)
+            if staticPackage[-1] == str(Globals.OS_SEPARATOR) :
+                staticPackage = staticPackage[:-1]
+            herokuPythonLibPath = Globals.HEROKU_PYTHON.replace(Globals.TOKEN_PYTHON_VERSION, str(self.getSetting(AttributeKey.PYTHON_VERSION)))
+            if staticPackage.endswith(herokuPythonLibPath) :
+                staticPackage = staticPackage.replace(herokuPythonLibPath,c.NOTHING)
+            staticPackage = f'{staticPackage}{Globals.STATIC_PACKAGE_PATH}'
+            self.setting(f'Static package (after handlings): "{staticPackage}"')
+        else :
+            self.setting(f'Static package (taken from application-env.yml): "{staticPackage}"')
         return staticPackage
 
     def log(self,message,exception=None):
@@ -624,23 +632,37 @@ class AttributeKey:
     KW_UPDATE = 'update'
     KW_RESOURCE = 'resource'
 
-    GLOBALS_API_LIST = f'{KW_API}.{KW_LIST}'
+    GLOBALS_API_LIST = f'{KW_API}{c.DOT}{KW_LIST}'
 
-    API_NAME = f'{KW_API}.{KW_NAME}'
-    API_EXTENSION = f'{KW_API}.{KW_EXTENSION}'
+    API_NAME = f'{KW_API}{c.DOT}{KW_NAME}'
+    API_EXTENSION = f'{KW_API}{c.DOT}{KW_EXTENSION}'
     UPDATE_GLOBALS = f'{KW_UPDATE}-globals'
     PRINT_STATUS = 'print-status'
-    DEPENDENCY_UPDATE = f'{KW_API}.{KW_DEPENDENCY}.{KW_UPDATE}'
-    DEPENDENCY_LIST_WEB = f'{KW_API}.{KW_DEPENDENCY}.{KW_LIST}.{KW_WEB}'
-    DEPENDENCY_LIST_LOCAL = f'{KW_API}.{KW_DEPENDENCY}.{KW_LIST}.{KW_LOCAL}'
-    DEPENDENCY_RESOURCE_LIST = f'{KW_API}.{KW_DEPENDENCY}.{KW_LIST}.{KW_LOCAL}'
-    PYTHON_VERSION = 'python.version'
+    DEPENDENCY_UPDATE = f'{KW_API}{c.DOT}{KW_DEPENDENCY}{c.DOT}{KW_UPDATE}'
+    DEPENDENCY_LIST_WEB = f'{KW_API}{c.DOT}{KW_DEPENDENCY}{c.DOT}{KW_LIST}{c.DOT}{KW_WEB}'
+    DEPENDENCY_LIST_LOCAL = f'{KW_API}{c.DOT}{KW_DEPENDENCY}{c.DOT}{KW_LIST}{c.DOT}{KW_LOCAL}'
+    DEPENDENCY_RESOURCE_LIST = f'{KW_API}{c.DOT}{KW_DEPENDENCY}{c.DOT}{KW_LIST}{c.DOT}{KW_LOCAL}'
+    PYTHON = 'python'
+    PYTHON_VERSION = f'{PYTHON}{c.DOT}version'
+    PYTHON_STATIC_PACKAGE = f'{PYTHON}{c.DOT}static-package'
 
     def getKey(api,key):
-        return f'{Globals.__name__}.{key}'
+        return f'{Globals.__name__}{c.DOT}{key}'
 
     def getKeyByClassNameAndKey(cls,key):
-        return f'{cls.__name__}.{key}'
+        return f'{cls.__name__}{c.DOT}{key}'
+
+def getResourceNameList(resourceNameList) :
+    return resourceNameList if ObjectHelper.isList(resourceNameList) else [resourceNameList]
+
+def getResourceName(resourceName) :
+    return resourceName if not c.DOT in resourceName else getResourceNameList(resourceName.split(c.DOT))[0]
+
+def getInnerResourceNameList(resourceName, resourceModuleName) :
+    if not resourceName == resourceModuleName :
+        resourceNameList = getResourceNameList(resourceName.split(c.DOT))
+        return [resourceNameList[0]] if 1 == len(resourceNameList) or resourceNameList[1] is None else resourceNameList
+    return [resourceName]
 
 def importModule(resourceModuleName, muteLogs=False, ignoreList=IGNORE_MODULE_LIST) :
     if resourceModuleName not in ignoreList :
@@ -658,14 +680,17 @@ def importModule(resourceModuleName, muteLogs=False, ignoreList=IGNORE_MODULE_LI
         return module
 
 def importResource(resourceName, resourceModuleName=None, muteLogs=False, ignoreList=IGNORE_REOURCE_LIST) :
-    if resourceName not in ignoreList :
-        if ObjectHelper.isEmpty(resourceModuleName) :
-            resourceModuleName = resourceName
-        module = importModule(resourceModuleName, muteLogs=False)
+    innerResourceName = getResourceName(resourceName)
+    if innerResourceName not in ignoreList :
+        if ObjectHelper.isNone(resourceModuleName) :
+            resourceModuleName = innerResourceName
+        module = importModule(resourceModuleName, muteLogs=muteLogs)
         if module :
             resource = None
             try :
-                resource = getattr(module, resourceName)
+                resource = module
+                for name in getInnerResourceNameList(resourceName, resourceModuleName) :
+                    resource = ReflectionHelper.getAttributeOrMethod(resource, name)
             except Exception as exception :
                 if not muteLogs :
                     log.failure(importResource, f'Not possible to import "{resourceName}" resource from "{resourceModuleName}" module', exception=exception)
